@@ -27,6 +27,10 @@ class DummyDeviceControl(DeviceControlInterface):
     def delete_args(self):
         return self._delete_args
 
+    @property
+    def wifi_args(self):
+        return self._wifi_args
+
     @property
     def wifi_connect_args(self):
         return self._wifi_connect_args
@@ -53,10 +57,11 @@ class DummyDeviceControl(DeviceControlInterface):
         self._result_show_all = result_show_all or []
         self._result_wifi_hotspot = result_wifi_hotspot
         self._show_args: List[str] = []
-        self._connect_args: List[str] = []
-        self._disconnect_args: List[str] = []
+        self._connect_args: List[Tuple] = []
+        self._disconnect_args: List[Tuple] = []
         self._reapply_args: List[str] = []
-        self._delete_args: List[str] = []
+        self._delete_args: List[Tuple] = []
+        self._wifi_args: List[str] = []
         self._wifi_connect_args: List[Tuple] = []
         self._wifi_hotspot_args: List[Tuple] = []
         self._wifi_rescan_args: List[Tuple] = []
@@ -80,32 +85,35 @@ class DummyDeviceControl(DeviceControlInterface):
         self._raise_error_if_needed()
         return self._result_show_all
 
-    def connect(self, ifname: str) -> None:
+    def connect(self, ifname: str, wait: int = None) -> None:
         self._raise_error_if_needed()
-        self._connect_args.append(ifname)
+        self._connect_args.append((ifname, wait))
 
-    def disconnect(self, ifname: str) -> None:
+    def disconnect(self, ifname: str, wait: int = None) -> None:
         self._raise_error_if_needed()
-        self._disconnect_args.append(ifname)
+        self._disconnect_args.append((ifname, wait))
 
     def reapply(self, ifname: str) -> None:
         self._raise_error_if_needed()
         self._reapply_args.append(ifname)
 
-    def delete(self, ifname: str) -> None:
+    def delete(self, ifname: str, wait: int = None) -> None:
         self._raise_error_if_needed()
-        self._delete_args.append(ifname)
+        self._delete_args.append((ifname, wait))
 
-    def wifi(self) -> List[DeviceWifi]:
+    def wifi(self, ifname: str = None) -> List[DeviceWifi]:
         self._raise_error_if_needed()
+        if ifname is not None:
+            self._wifi_args.append(ifname)
         return self._result_wifi
 
-    def wifi_connect(self, ssid: str, password: str, ifname: str = None) -> None:
+    def wifi_connect(self,
+                     ssid: str,
+                     password: str,
+                     ifname: str = None,
+                     wait: int = None) -> None:
         self._raise_error_if_needed()
-        if ifname is None:
-            self._wifi_connect_args.append((ssid, password))
-        else:
-            self._wifi_connect_args.append((ssid, password, ifname))
+        self._wifi_connect_args.append((ssid, password, ifname, wait))
 
     def wifi_hotspot(self,
                      ifname: str = None,
